#!/usr/bin/perl
# Tests /admin/*/list pages

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More;

# Expected results
my $test_types = {
    Role    => { count => 14,  search => { count  => 2,   string => "all" } },
    Contact => { count => 14, search => { string => "o", count  => 13 } },
    Hostgroup => { count => 7, name => "Host Group", search => { string => "eaf", count => 2 } },
};

plan tests => 8 * ( scalar keys %$test_types );

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use Opsview::Test;
use Log::Log4perl;
use Log::Log4perl::Level;
# reduce unnecessary output by amending rootlogger to suppress INFO and DEBUG messages
Log::Log4perl->get_logger("")->level($WARN);

foreach my $type ( keys %$test_types ) {
    my $mech = Test::WWW::Mechanize::Catalyst->new;

    $mech->get_ok( "http://localhost/admin/" . lc($type) );

    $mech->content_like( qr/login_username/, "Login page" );

    $mech->form_name('login_form');
    $mech->field( 'login_username' => "admin" );
    $mech->field( 'login_password' => "initial" );
    $mech->field( "noscript"       => 0 );
    $mech->click('login');

    my $type_name = $test_types->{$type}->{name} || $type;
    $mech->content_like(qr/Create new $type_name/);
    $mech->title_is("Opsview > $type_name > List");

    my $rows;
    $rows = grep /Edit /, split( "\n", $mech->content );
    my $all_count = $test_types->{$type}->{count};
    is( $rows, $all_count, "Got $all_count $type" );

    $mech->form_name("search_list");
    my $search_string = $test_types->{$type}->{search}->{string};
    $mech->field( "q", $search_string );
    $mech->submit_form();
    $rows = grep /Edit /, split( "\n", $mech->content );
    my $search_count = $test_types->{$type}->{search}->{count};
    is( $rows, $search_count, "Got $search_count $type from search of '$search_string' " );

    $mech->get_ok( "http://localhost/admin/" . lc($type) . "?rows=1" );
    my $link = $mech->find_link( text => $all_count );
    like( $link->url, "/page=$all_count/", "When rows=1, lists all pages" );
    1;
}

1;
