#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
#use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use Test::WWW::Mechanize;

use lib "$Bin/lib";
use Opsview::TestUtils;

use Data::Dump qw(dump);

my $content;
#my $ua = Test::WWW::Mechanize::Catalyst->new( stack_depth => 0 );
my $ua = Test::WWW::Mechanize->new( stack_depth => 0 );

my $res_data;

rest_log_in( $ua, "admin", "initial", "http://localhost:3000" );

my $hostgroup_data_json = qq%
{
   "summary" : {
      "allrows" : "12",
      "rows" : "12"
   },
   "list" : [
      {
         "parent" : null,
         "matpath" : "Opsview,",
         "ref" : "/rest/config/hostgroup/1",
         "name" : "Opsview",
         "id" : "1"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/1",
            "name" : "Opsview",
            "matpath" : "Opsview,"
         },
         "matpath" : "Opsview,alphaearly,",
         "ref" : "/rest/config/hostgroup/9",
         "name" : "alphaearly",
         "id" : "9"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/1",
            "name" : "Opsview",
            "matpath" : "Opsview,"
         },
         "matpath" : "Opsview,Passive Monitoring,",
         "ref" : "/rest/config/hostgroup/10",
         "name" : "Passive Monitoring",
         "id" : "10"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/1",
            "matpath" : "Opsview,",
            "name" : "Opsview"
         },
         "matpath" : "Opsview,Production,",
         "ref" : "/rest/config/hostgroup/11",
         "name" : "Production",
         "id" : "11"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/11",
            "matpath" : "Opsview,Production,",
            "name" : "Production"
         },
         "matpath" : "Opsview,Production,UK,",
         "ref" : "/rest/config/hostgroup/3",
         "name" : "UK",
         "id" : "3"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/3",
            "matpath" : "Opsview,Production,UK,",
            "name" : "UK"
         },
         "matpath" : "Opsview,Production,UK,Leaf,",
         "ref" : "/rest/config/hostgroup/4",
         "name" : "Leaf",
         "id" : "4"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/3",
            "matpath" : "Opsview,Production,UK,",
            "name" : "UK"
         },
         "matpath" : "Opsview,Production,UK,Monitoring Servers,",
         "ref" : "/rest/config/hostgroup/2",
         "name" : "Monitoring Servers",
         "id" : "2"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/1",
            "matpath" : "Opsview,",
            "name" : "Opsview"
         },
         "matpath" : "Opsview,singlehost,",
         "ref" : "/rest/config/hostgroup/7",
         "name" : "singlehost",
         "id" : "7"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/1",
            "matpath" : "Opsview,",
            "name" : "Opsview"
         },
         "matpath" : "Opsview,Test,",
         "ref" : "/rest/config/hostgroup/12",
         "name" : "Test",
         "id" : "12"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/12",
            "matpath" : "Opsview,Test,",
            "name" : "Test"
         },
         "matpath" : "Opsview,Test,UK,",
         "ref" : "/rest/config/hostgroup/6",
         "name" : "UK",
         "id" : "6"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/6",
            "matpath" : "Opsview,Test,UK,",
            "name" : "UK"
         },
         "matpath" : "Opsview,Test,UK,Leaf2,",
         "ref" : "/rest/config/hostgroup/5",
         "name" : "Leaf2",
         "id" : "5"
      },
      {
         "parent" : {
            "ref" : "/rest/config/hostgroup/6",
            "matpath" : "Opsview,Test,UK,",
            "name" : "UK"
         },
         "matpath" : "Opsview,Test,UK,middling,",
         "ref" : "/rest/config/hostgroup/8",
         "name" : "middling",
         "id" : "8"
      }
   ]
}
%;
use JSON;
my $json = JSON->new->relaxed(1);
my $hostgroup_data = $json->decode($hostgroup_data_json);

$ua->add_header( 'content-type' => "application/json" );

$ua->put( "/rest/config/hostgroup", Content => $hostgroup_data_json );
is( $ua->status, 200, "Can update hostgroup hierarchy");
$res_data = $json->decode( $ua->content );
is( $res_data->{objects_updated}, 12, "Got correct message" ) || diag( "Full content: ".$ua->content );

$ua->get_ok( "/rest/config/hostgroup?order=dependency&cols=-hosts,-children,-is_leaf,-uncommitted&rows=all" );
$res_data = $json->decode( $ua->content );
is_deeply( $res_data, $hostgroup_data, "Got all host group data back again" ) || diag explain $res_data;

$ua->put( "/rest/config/hostgroup", Content => $hostgroup_data_json );
is( $ua->status, 200, "Can resync same hostgroup hierarchy");
$res_data = $json->decode( $ua->content );
is( $res_data->{objects_updated}, 12, "Got correct message" ) || diag( "Full content: ".$ua->content );


# Move Test hostgroup to alphaearly
$ua->put( "/rest/config/hostgroup/6?cols=id,name,parent", Content => qq%{"parent" : { "name":"alphaearly" } }% );
is( $ua->status, 200, "Moved parent of Test host group happily to alphaearly" );
$res_data = $json->decode( $ua->content );
my $expected = {
    name => "UK",
    id => 6,
    parent => { 
        name => "alphaearly",
        matpath => "Opsview,alphaearly,",
        "ref" => "/rest/config/hostgroup/9",
    }
};
is_deeply( $res_data->{object}, $expected, "Moved host group to different part of tree" ) || diag( $ua->content );


# Try resync again
$ua->put( "/rest/config/hostgroup", Content => $hostgroup_data_json );
is( $ua->status, 200, "Can resync same hostgroup hierarchy");
$res_data = $json->decode( $ua->content );
is( $res_data->{objects_updated}, 12, "Got correct message" ) || diag( "Full content: ".$ua->content );

# Confirm UK moved back
$ua->get_ok( "/rest/config/hostgroup/6?cols=id,name,parent");
$res_data = $json->decode( $ua->content );
$expected = {
    name => "UK",
    id => 6,
    parent => { 
        "ref" => "/rest/config/hostgroup/12",
        "matpath" => "Opsview,Test,",
        "name" => "Test"
    }
};
is_deeply( $res_data->{object}, $expected, "Host group hierarchy reverted back" ) || diag( $ua->content );

