#!/usr/bin/perl
# Test ACLs

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Opsview::Test;
use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use lib "$Bin/lib";
use Opsview::TestUtils;

my $admin = Test::WWW::Mechanize::Catalyst->new;
log_in( $admin, 'admin', 'initial' );

my $demo = Test::WWW::Mechanize::Catalyst->new;
log_in( $demo, 'demo', 'demo' );

my $somehosts = Test::WWW::Mechanize::Catalyst->new;
log_in( $somehosts, "somehosts", "somehosts" );

my $readonly = Test::WWW::Mechanize::Catalyst->new;
log_in( $readonly, "readonly", "readonly" );

my $viewsomechangenone = Test::WWW::Mechanize::Catalyst->new;
log_in( $viewsomechangenone, "viewsomechangenone", "viewsomechangenone" );

my $adminnoc = Test::WWW::Mechanize::Catalyst->new;
log_in( $adminnoc, "adminnoc", "adminnoc" );

my $public = Test::WWW::Mechanize::Catalyst->new;



# Check demo role
$demo->get_ok("/admin/contact");
$demo->title_is("Opsview > Contact > List");
$demo->follow_link( text => "adminnoc" );
$demo->title_is("Opsview > Contact > Edit: adminnoc");
$demo->content_lacks('type="submit"', "Missing the submit button, as expected");
$demo->form_name("main_form");
$demo->submit_form;
is( $demo->status, 403, "Save access denied" );

$demo->get("/admin/contact/delete/4");
is( $demo->status, 403, "Delete denied" );

$demo->get("/admin/host/query_host");
is( $demo->status, 403, "Query host blocked" );

$demo->get("/admin/notificationmethods/set_order");
is( $demo->status, 403, "Reordering blocked" );

$demo->get("/admin/snmpwalk/poll");
is( $demo->status, 403, "snmpwalk blocked" );

$demo->get("/admin/ajax/send_test_sms");
is( $demo->status, 403, "send_test_sms blocked");

$demo->get("/admin/ajax/check_credit");
is( $demo->status, 403, "check_credit blocked");

$demo->get("/admin/systempreference/displaysurveydata");
is( $demo->status, 403, "displaysurveydata blocked");

$demo->get("/admin/auditlog");
is( $demo->status, 403, "auditlog blocked" );

$demo->get_ok("/user/preference");
$demo->content_lacks("password", "No password field should be available");

$demo->post_ok( "/user/preference/save", { password => "changed" }, "Check post is ignored" );

$demo->post( "/admin/host/save_new", { cloned_from => 1 } );
is( $demo->status, 403, "Blocked code since no CONFIGURESAVE" );

$demo->get_ok("/logout");
log_in( $demo, 'demo', 'demo' );
$demo->get_ok("/user/preference", "Check can still login and access user preference with old password");

$Opsview::Web::Controller::Common::rrddir = "$Bin/var/rrd";

# Set public role to not have RRDGRAPHS access
my $schema = Opsview::Schema->my_connect;

# Reset all RRDGRAPH access
my $public_role = $schema->resultset("Roles")->find(1);
$schema->resultset("Roles")->synchronise( { id => $public_role->id, accesses => [] } );
my $auth_role = $schema->resultset("Roles")->find(2);
$schema->resultset("Roles")->synchronise( { id => $auth_role->id, accesses => [] } );

$public->get_ok("/rrdgraph?metric=metricA&service=serviceX&host=host1");
$public->content_contains("login_username");
$public->get_ok("/graph");
$public->content_contains("login_username");
$admin->get("/rrdgraph?metric=metricA&service=serviceX&host=host1");
is( $admin->status, 403, "/rrdgraph access denied if already logged in" );
$admin->get("/graph");
is( $admin->status, 403, "/graph access denied if already logged in" );

$schema->resultset("Roles")->synchronise( { id => $auth_role->id, accesses => [ { name => "RRDGRAPHS" } ] } );

$admin->get_ok("/rrdgraph?metric=metricA&service=serviceX&host=host1");
is( $admin->content_type, "image/png", "Got rrdgraph as png" );
$admin->get_ok("/graph");
$admin->content_contains("graph");
$public->get_ok("/rrdgraph?metric=metricA&service=serviceX&host=host1");
$public->content_contains( "login_username", "/rrdgraph denied for public users" );
$public->get_ok("/graph");
$public->content_contains( "login_username", "/graph denied for public users" );

$schema->resultset("Roles")->synchronise( { id => $public_role->id, accesses => [ { name => "RRDGRAPHS" } ] } );

$public->get_ok("/rrdgraph?metric=metricA&service=serviceX&host=host1");
is( $public->content_type, "image/png", "Can get public rrdgraph" );
$public->get_ok("/graph?host=opsviewdev1&service=HTTP");
# Not entirely sure why, but this is Graph > Summary, rather than Graph > opsviewdev1::HTTP
# I think due to changing the rrd paths within the startup dev server
$public->content_contains("Graph &gt; Summary");
$public->content_contains("Not logged in");
$public->content_lacks('class="hsearch"');
$public->get_ok( "/pub/ylabel.svg", "Getting ylabel SVG (can't follow link because done via JS" );
$public->content_contains( "<svg", "Has svg content" );

$public->get_ok("/rrdfetch?start=1229598294&end=1261134294&hsm=opsviewdev1::HTTP::size");
$public->content_is('{"ResultSet":{"lines":[]}}');

$schema->resultset("Roles")->synchronise( { id => $public_role->id, accesses => [ ] } );
$schema->resultset("Roles")->synchronise( { id => $auth_role->id, accesses => [ ] } );

$public->get_ok("/viewport");
$public->content_contains( "login_username", "Still need auth for viewports" );

$admin->get("/viewport");
is( $admin->status, 403, "Access denied for logged in users" );

$schema->resultset("Roles")->synchronise( { id => $auth_role->id, accesses => [ { name => "VIEWPORTACCESS" } ] } );
$admin->get_ok("/viewport");
$admin->content_lacks( "login_username", "OK now for viewports!" );

$public->get_ok("/viewport");
$public->content_contains( "login_username", "Need auth for viewports" );

$schema->resultset("Roles")->synchronise( { id => $public_role->id, accesses => [ { name => "VIEWPORTACCESS" } ] } );
$public->get_ok("/viewport");
$public->content_lacks( "login_username", "OK now for viewports!" );

$admin->get_ok("/admin/host");
my $rows = grep /Edit /, split( "\n", $admin->content );
is( $rows, 24 );
$admin->content_contains( "Cisco Mgt", "Host templates displayed" );

$somehosts->get_ok("/admin/host");
$rows = grep /Edit /, split( "\n", $somehosts->content );
is( $rows, 5 );

$readonly->get("/admin/host");
is( $readonly->status, 403, "Forbidden" );

$admin->get_ok("/admin/host/edit/14");
$somehosts->get("/admin/host/edit/14");

is( $somehosts->status, 403, "Forbidden to host 14" );

$admin->get_ok("/admin/host/edit/12");
$admin->content_contains( "ClusterA",                 "Got slave cluster" );
$admin->content_contains( "Master Monitoring Server", "And master monitoring server" );
$admin->content_contains( "hostgroup_new",            "And hostgroup option" );
$admin->content_contains( "Template details",         "And template details" );

$somehosts->get_ok("/admin/host/edit/4");
$somehosts->content_contains( "ClusterA", "Got slave cluster" );
$somehosts->content_lacks( "Master Monitoring Server", "And no master monitoring server" );
$somehosts->content_lacks( "hostgroup_new",            "And no new hostgroup option" );
$somehosts->content_lacks( "Template details",         "No template details because no permissions" );
$somehosts->form_name("main_form");
$somehosts->field("alias", "Changed via test" );
$somehosts->submit_form;
is( $somehosts->status, 200, "Should save hosts happily" );

$somehosts->get_ok("/admin/host/edit/4");
$somehosts->form_name("main_form");
is( $somehosts->value("alias"), "Changed via test", "Submission okay" );

$somehosts->get("/admin/servicecheck");
is( $somehosts->status, 403, "servicecheck forbidden" );

$somehosts->post("/admin/servicecheck/save/4", { name => "changed name", description => "changed description" } );
is( $somehosts->status, 403, "forbidden from saving" );

$somehosts->get_ok("/admin/ajax/options_for_parent_hosts");
my $data = $somehosts->content;
$somehosts->get_ok("/admin/ajax/options_for_parent_hosts?monitoringserver_filter=2");
is( $somehosts->content, $data, "Same data as always filtered by monitoringserver" );
$somehosts->get_ok("/admin/ajax/options_for_parent_hosts?monitoringserver_filter=1");
is( $somehosts->content, $data, "Again, same as ms=2" );

# Check that adminnoc (no CONFIGUREHOSTS), does not see hosts
$adminnoc->get("/admin/host/edit/4");
is( $adminnoc->status, 403, "Cannot see hosts" );
$adminnoc->post("/admin/host/save/4", { name => "blubber", alias => "Edit 4" } );
is( $adminnoc->status, 403, "Cannot save" );

# admin should still be able to change keywords
$admin->get_ok("/admin/keyword/edit/7");
$admin->form_name("main_form");
$admin->field("name", "allhosts-changed");
$admin->submit_form;
$admin->get_ok("/admin/keyword/edit/7");
$admin->form_name("main_form");
is( $admin->value("name"), "allhosts-changed", "Changed keyword name");

# adminnoc does not have CONFIGUREKEYWORDS, so prove is blocked
$adminnoc->get("/admin/keyword/edit/7");
is( $adminnoc->status, 403, "No access to keyword edit page" );
$adminnoc->get("/admin/keyword/delete/7");
is( $adminnoc->status, 403, "Cannot delete keyword either" );
$adminnoc->get("/admin/keyword/new");
is( $adminnoc->status, 403, "Cannot add new keyword either" );

$admin->get_ok("/admin/ajax/options_for_parent_hosts");
isnt( $admin->content, $data, "More data for admin user as more hosts listed" );

$admin->get_ok("/admin/servicegroup/services/1?host=12");
$somehosts->get_ok("/admin/servicegroup/services/1?host=12");
$somehosts->get("/admin/servicegroup/services/1?host=14");
is( $somehosts->status, 403, "Forbidden for this host" );

$somehosts->get_ok("/admin/snmpwalk/poll?hostid=12");
is( $somehosts->content, '{"ResultSet":{"state":"start"}}' );
$somehosts->get("/admin/snmpwalk/poll?hostid=14");
is( $somehosts->status, 403, "Forbidden for this host" );

$somehosts->get("/info/host/12");
is( $somehosts->status, 500, "Get web exception" );
TODO: {
    local $TODO = "Bug: OPS-317 - host info error for new hosts";
    $somehosts->content_lacks( "Caught exception", "No error on this page" );
    $somehosts->content_contains( 'stuff', "Got info for new host" );
}
$somehosts->get_ok("/info/host/6");
$somehosts->content_like( qr/No comments currently entered for this item/, "Got info for existing host" );
$somehosts->get("/info/host/14");
is( $somehosts->status, 403, "Forbidden for this host" );

$viewsomechangenone->get_ok("/info/host/3");
$viewsomechangenone->content_lacks( "Edit", "No edit link" );
my $content = $viewsomechangenone->content;
$viewsomechangenone->get_ok("/info/host/3/edit");
is( $viewsomechangenone->content, $content, "Exactly same output as edit redirects to info for non admin users" );
$viewsomechangenone->get("/info/host/1");
is( $viewsomechangenone->status, 403, "Forbidden to look at opsview host" );
$viewsomechangenone->get_ok("/info/host/56356356");
$viewsomechangenone->content_contains("Id 56356356 not found");

$admin->get_ok("/info/host/3");
$admin->content_contains( "Edit", "Can edit host info page" );

$public->get("/public/closest_monitoringserver/hostname");
$public->content_contains( "login_username", "Got login page" );

$public->get_ok("/admin/reload");
$public->content_contains( "login_username", "Got login page for public at /admin/reload" );

$somehosts->get("/admin/reload");
is( $somehosts->status, 403, "Denied reload to somehosts" );
$somehosts->get("/admin/reload/do_reload");
is( $somehosts->status, 403, "Denied do_reload to somehosts" );
$somehosts->get("/admin/reload/do_restart");
is( $somehosts->status, 403, "Denied do_restart to somehosts" );
$somehosts->get("/admin/reload/config_errors");
is( $somehosts->status, 403, "Denied config_errors to somehosts" );

$somehosts->get("/admin/status/opsview");
is( $somehosts->status, 403, "Denied opsview status to somehosts" );

$admin->get_ok("/admin/reload");
$admin->content_contains( "Opsview", "admin user can reload" );
$admin->get_ok("/admin/status/opsview");
$admin->content_contains("<status>");

# Do this test last, as it removes the whole RELOADACCESS access
$schema->resultset("Access")->search( { name => "RELOADACCESS" } )->delete_all;
$admin->get("/admin/reload");
is( $admin->status, 403, "Denied reload" );
$admin->get("/admin/status/opsview");
is( $admin->status, 403, "Denied opsview status without RELOADACCESS" );

my $viewuser = Test::WWW::Mechanize::Catalyst->new;
log_in( $viewuser, "viewsomechangenonewonotify", "viewsomechangenonewonotify" );

my $view_role = $schema->resultset("Contacts")->find( { name => "viewsomechangenonewonotify" } )->role;
$schema->resultset("Roles")->synchronise( { id => $view_role->id, accesses => [ { name => "NOTIFYSOME" } ] } );
$viewuser->get("/event");
is( $viewuser->status, 403, "Denied event view because no VIEWSOME nor VIEWALL" );

$viewuser->get("/search/performancemetric");
is( $viewuser->status, 403, "Denied no VIEWSOME nor VIEWALL" );

$schema->resultset("Roles")->synchronise( { id => $view_role->id, accesses => [ { name => "VIEWSOME" } ] } );
$viewuser->get("/event");
is( $viewuser->status, 200, "Can now see because of VIEWSOME" );

$schema->resultset("Roles")->synchronise( { id => $view_role->id, accesses => [ { name => "VIEWALL" } ] } );
$viewuser->get("/event");
is( $viewuser->status, 200, "Can now see because of VIEWALL" );

$schema->resultset("Roles")->synchronise( { id => $view_role->id, accesses => [ { name => "VIEWALL" }, { name => "VIEWSOME" } ] } );
$viewuser->get("/event");
is( $viewuser->status, 200, "Can still see with both VIEWSOME and VIEWALL" );

my $freshuser = Test::WWW::Mechanize::Catalyst->new;

$freshuser->get_ok("/event");
is( $freshuser->status, 200, "Getting 200 for login form when trying to access /event" );
$freshuser->content_contains( "login_form", "And login stuff here" );

$public->get("/search/host?q=s");
# Don't get a 403 - just get redirected to login page now
#is( $public->status, 403, "Denied to search pages" );
$public->content_contains("login_username");

$public->get("/admin/snmptrapexception");
$public->content_contains("login_username");

$viewuser->get("/admin/snmptrapexception");
is( $viewuser->status, 403, "Access denied for viewsome user" );
$viewuser->content_contains("Logged in as");
