#!/usr/bin/perl
# Test keyword edit pages

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize;

use lib "$Bin/lib";
use Opsview::TestUtils;

use Data::Dump qw(dump);

my $content;
my $ua = Test::WWW::Mechanize->new;
my $admin = Test::WWW::Mechanize->new;

use JSON;
my $json = JSON->new;

log_in( $ua, 'keywordonly', 'keywordonly', "http://localhost:3000" );
log_in( $admin, "admin", "initial", "http://localhost:3000" );

my $link = $ua->find_link( text => "Keywords" );
is( $link->url, "/admin/keyword/list", "Link for keywords is seen" );

$ua->get_ok("/admin/keyword/edit/8");
$ua->form_name("main_form");
is( $ua->value(".filter_by_hosts"), undef, "Filter button unticked on load" );

$ua->content_lacks("Keywordonly</option>", "Keywordonly, a role with all_keywords, should not get listed");
$ua->content_contains("<li>Keywordonly</li>", "...but will get listed for information" );
$ua->content_contains("all keywords", "Some info text re: roles with all keywords" );
$ua->content_contains("Additional Roles:</a>", "Has label with 'Additional Roles'");

$ua->get_ok("/admin/keyword/ajax_servicechecks?hosts=9&hosts=7&.filter_by_hosts=1");
my $res = $json->decode($ua->content);
my $expected = {
  ResultSet => {
        Results => [
              { name => "Another exception", value => 82 },
              { name => "Check Memory", value => 44 },
              { name => "Coldstart", value => 79 },
              { name => "Interface", value => 95 },
              { name => "snmp poll", value => 83 },
              { name => "Test exceptions", value => 81 },
           ],
      },
};
is_deeply($res, $expected, "Got expected json data") || diag(dump($res));

$ua->get_ok("/admin/keyword/list");
$ua->content_lacks("newkeyword", "No newkeyword created");

$ua->get_ok("/admin/keyword/new");
$ua->form_name("main_form");
$ua->field("name", "newkeyword");
is($ua->value("enabled"), 0, "Viewport disabled initially");
$ua->field("enabled", 1);
$ua->tick("public", 1);
$ua->submit_form;

$ua->get_ok("/admin/keyword/list");
$ua->content_contains("newkeyword", "Found new keyword created");
$ua->follow_link_ok( { text => "newkeyword" } );

$ua->content_contains("Edit: newkeyword");
$ua->form_name("main_form");
is($ua->value("enabled"), 1, "Viewport now enabled");
is($ua->value("public"), 1, "public now ticked" );
$ua->field("enabled", 0);
$ua->untick("public", 1);
$ua->submit_form;

$ua->get_ok("/admin/keyword/list");
$ua->follow_link_ok( { text => "newkeyword" } );
$ua->content_contains("Edit: newkeyword");
$ua->form_name("main_form");
is($ua->value("enabled"), 0, "Viewport now disabled");
is($ua->value("public"), undef, "public now unticked" );

$admin->get_ok("/admin/role/edit/21");
$admin->form_name("main_form");
$admin->untick("all_keywords",1);
$admin->submit_form;

$ua->get_ok("/admin/keyword/edit/8");
$ua->content_contains("Keywordonly</option>", "Now Keywordonly gets listed because not all_keywords set");
$ua->content_lacks("<li>Keywordonly</li>", "...and not listed here" );
$ua->content_lacks("all keywords", "Some info text re: roles with all keywords" );
$ua->content_contains(">Roles:</a>", "Has label with 'Roles'");

