#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use lib "$Bin/lib";
use Opsview::TestUtils;

my $mech = Test::WWW::Mechanize::Catalyst->new;

log_in( $mech, 'admin', 'initial' );

$mech->get_ok("/admin/notificationprofile");
$mech->content_like(qr/Contact does not exist/);

$mech->get_ok("/admin/notificationprofile?contact=2");
$mech->content_like(qr/Default/, "Got list page");

$mech->follow_link( text => "Default");
my $url = $mech->uri;
$mech->get_ok( $url, "Go back to page again, to test referrer" );
$mech->form_name("main_form");
is( $mech->value( "all_hostgroups" ), 1, "all_hostgroups is ticked" );
is( $mech->value( "all_servicegroups"), 1, "all_servicegroups ticked" );

$mech->untick("all_hostgroups", 1);
$mech->submit_form;

is($mech->status, 200);
$mech->content_contains(qq{<meta http-equiv="refresh" content="0; URL=http://localhost/admin/notificationprofile?contact=2" />}, "Goes back to list page");

$mech->get_ok( $url, "Go back to edit page again" );
$mech->form_name("main_form");
is( $mech->value( "all_hostgroups" ), undef, "all_hostgroups is unticked" );
is( $mech->value( "all_servicegroups"), 1, "all_servicegroups remains ticked" );


$mech->get_ok("/admin/notificationprofile?contact=8");
$mech->follow_link( text => "Default");

$mech->content_contains( "Leaf2", "Found host group this contact has access to" );
$mech->content_lacks( "middling", "...and this host group is not listed correctly" );

$mech->content_contains( "Operations", "Similarly for service group" );
$mech->content_lacks( "freshness checks", "...and also for a missing service group" );

$mech->content_contains( "alphaearly", "Keywords" );
$mech->content_lacks( "cloneable", "...and missing keyword" );

# Test uniqueness of names,contact
$mech->get_ok("/admin/notificationprofile/unique?contact=10&name=Non+work+hours+sms");
is( $mech->content, '{"ResultSet":1}' );

$mech->get_ok("/admin/notificationprofile/unique?contact=10&name=High+priority");
is( $mech->content, '{"ResultSet":0}' );

1;
