#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/../lib";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/t/lib";

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use Sys::Hostname;

use lib "$Bin/lib";
use Opsview::Web::Test;

use Test::More qw(no_plan);

my $ua = WWW::Mechanize->new();

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

my $res;

# Check /rss gives a deprecated message
$res = $ua->get("http://$hostname/rss");
ok( $res->is_success, "http://$hostname/rss - success and..." );
is( $res->code, 200, "Status 200 - but ..." );
like( $res->content, "/rss version.*Opsview RSS feed has been deprecated/s", "Deprecated - message as expected" );

# Have to use LWP::UserAgent here, because WWW::Mechanize will continue to retry and then dies
# because of the number of iterations
$ua  = LWP::UserAgent->new();
$res = $ua->get("http://$hostname/atom");
ok( $res->is_error, "http://$hostname/atom - error" );
is( $res->code, 401, "Status 401 - authorization error" );
like( $res->content, "/Authorization required/", "Auth error" );

$ua  = LWP::UserAgent->new();
$ua->credentials( $hostname, "Nagios Access", "admin", "badpassword" );
$res = $ua->get("http://$hostname/atom");
ok( $res->is_error, "http://$hostname/atom - error with wrong password" );
is( $res->code, 401, "Status 401 - authorization error" );
like( $res->content, "/Authorization required/", "Auth error" );

$ua  = LWP::UserAgent->new();
$ua->credentials( $hostname, "Nagios Access", "admin", "initial" );
$res = $ua->get("http://$hostname/atom");
ok( $res->is_success, "http://$hostname/atom - success" );
is( $res->code, 200, "Status 200 - Got Atom feed" );
like( $res->content, "/xml version.*feed xmlns/s", "Data appears okay" );

use Opsview::TestUtils;
use Test::WWW::Mechanize;
my $ua_cookies = Test::WWW::Mechanize::Catalyst->new();
$ua_cookies->get("http://localhost/atom");
is( $ua_cookies->status, 401, "Authorization failed, as expected" );
log_in( $ua_cookies, "admin", "initial" );
$ua_cookies->get_ok("http://localhost/atom", "Can login using cookies");
$ua_cookies->content_like( qr/xml version.*feed xmlns/s, "Content looks okay" );

